>From 85798e0bb4a03f76cef39f3e16b3f641245985b6 Mon Sep 17 00:00:00 2001
From: Roiy <roiy@cs.umanitoba.ca>
Date: Fri, 4 Oct 2013 14:01:23 -0500
Subject: [PATCH 8/9] MED controller

---
 include/vlc_common.h                      |   1 +
 modules/gui/qt4/adapters/seeksegments.cpp | 121 +++++++++++++
 modules/gui/qt4/adapters/seeksegments.hpp | 100 +++++++++++
 modules/gui/qt4/main_interface.cpp        |   8 +
 modules/gui/qt4/main_interface.hpp        |   3 +
 modules/gui/qt4/styles/metastyle.cpp      | 275 ++++++++++++++++++++++++++++++
 modules/gui/qt4/styles/metastyle.hpp      |  38 +++++
 modules/gui/qt4/util/input_slider.cpp     | 258 +++++++++++++++++++++++++++-
 modules/gui/qt4/util/input_slider.hpp     |  69 +++++++-
 9 files changed, 865 insertions(+), 8 deletions(-)
 create mode 100644 modules/gui/qt4/adapters/seeksegments.cpp
 create mode 100644 modules/gui/qt4/adapters/seeksegments.hpp
 create mode 100644 modules/gui/qt4/styles/metastyle.cpp
 create mode 100644 modules/gui/qt4/styles/metastyle.hpp

diff --git a/include/vlc_common.h b/include/vlc_common.h
index 6116f8e..733803d 100644
--- a/include/vlc_common.h
+++ b/include/vlc_common.h
@@ -242,6 +242,7 @@ typedef struct es_out_t     es_out_t;
 typedef struct es_out_id_t  es_out_id_t;
 typedef struct es_out_sys_t es_out_sys_t;
 typedef struct seekpoint_t seekpoint_t;
+typedef struct seeksegment_t seeksegment_t;
 typedef struct info_t info_t;
 typedef struct info_category_t info_category_t;
 typedef struct input_attachment_t input_attachment_t;
diff --git a/modules/gui/qt4/adapters/seeksegments.cpp b/modules/gui/qt4/adapters/seeksegments.cpp
new file mode 100644
index 0000000..0f1db77
--- /dev/null
+++ b/modules/gui/qt4/adapters/seeksegments.cpp
@@ -0,0 +1,121 @@
+/*****************************************************************************
+ * seeksegments.cpp : Segments for MetaLayers
+ *****************************************************************************
+ * Copyright © 2011 the VideoLAN team
+ *
+ * Authors: Roiy Shpaner <roiy@cs.umanitoba.ca>
+ *
+ * This program is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation; either version 2 of the License, or
+ * ( at your option ) any later version.
+ *
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, write to the Free Software
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston MA 02110-1301, USA.
+ *****************************************************************************/
+
+
+#include "recents.hpp"
+#include "dialogs_provider.hpp"
+#include "menus.hpp"
+
+#include "seeksegments.hpp"
+
+#include "qt4.hpp"
+#include "input_manager.hpp"
+
+SeekSegments::SeekSegments( QObject *parent, intf_thread_t *p_intf_ ) :
+    QObject( parent ), p_intf( p_intf_ )
+{
+    psz_layerName = NULL;    
+}
+
+SeekSegments::~SeekSegments()
+{
+    free( psz_layerName );
+}
+
+
+void SeekSegments::update( bool isMain )
+{
+    input_thread_t *p_input = playlist_CurrentInput( THEPL );
+    seeksegment_t **seeksegments;
+    int res, i_seeksegments;
+
+    if( !p_input )
+        return;
+
+    // lock here too, as update event is triggered by an external thread
+    if ( !access() )
+    {
+        vlc_object_release( p_input );
+        return;
+    }
+
+    segmentsList.clear();
+
+    if ( isMain )
+        res = input_Control( p_input, INPUT_GET_SELECTED_SEGMENTS, &seeksegments, &i_seeksegments );
+    else
+        res = input_Control( p_input, INPUT_GET_SEGMENTS, &seeksegments, &i_seeksegments, psz_layerName );
+
+    vlc_object_release( p_input );
+
+    if ( res != VLC_SUCCESS )
+    {
+        release();
+        return;
+    }
+    
+    if ( i_seeksegments > 0 )
+    {
+        // first check the last point to see if we have filled time offsets (> 0)
+        if ( seeksegments[i_seeksegments - 1]->i_time_offset > 0 )
+        {
+            for ( int i=0; i < i_seeksegments ; i++ )
+                segmentsList << SeekSegment( seeksegments[i] );
+        }
+    }
+    msg_Dbg( p_intf, "ROIY Added Seek Segments");
+
+    for ( int j=0 ; j < i_seeksegments ; j++ )
+        vlc_seeksegment_Delete( seeksegments[j] );
+    free( seeksegments );
+    
+    release();
+}
+
+void SeekSegments::updateMain()
+{
+    update( true );
+}
+
+QList<SeekSegment> const SeekSegments::getSegments()
+{
+    QList<SeekSegment> copy;
+    if ( access() )
+    {
+        copy = segmentsList;
+        release();
+    }
+
+    return copy;
+}
+
+bool SeekSegments::jumpTo( int i_chapterindex )//ROIY REMOVE
+{
+    vlc_value_t val;
+    val.i_int = i_chapterindex;
+    input_thread_t *p_input_thread = playlist_CurrentInput( THEPL );
+    if( !p_input_thread ) return false;
+    bool b_succ = var_Set( p_input_thread, "chapter", val );
+    vlc_object_release( p_input_thread );
+    return ( b_succ == VLC_SUCCESS );
+}
+
diff --git a/modules/gui/qt4/adapters/seeksegments.hpp b/modules/gui/qt4/adapters/seeksegments.hpp
new file mode 100644
index 0000000..b7d1d01
--- /dev/null
+++ b/modules/gui/qt4/adapters/seeksegments.hpp
@@ -0,0 +1,100 @@
+/*****************************************************************************
+ * seekpoints.hpp : Segments for MetaLayers
+ *****************************************************************************
+ * Copyright © 2011 the VideoLAN team
+ *
+ * Authors: Roiy Shpaner <roiy@cs.umanitoba.ca>
+ *
+ * This program is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation; either version 2 of the License, or
+ * ( at your option ) any later version.
+ *
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, write to the Free Software
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston MA 02110-1301, USA.
+ *****************************************************************************/
+
+#ifndef SEEKSEGMENTS_HPP
+#define SEEKSEGMENTS_HPP
+
+#ifdef HAVE_CONFIG_H
+#include "config.h"
+#endif
+
+#include <vlc_common.h>
+#include <vlc_interface.h>
+#include <vlc_input.h>
+
+#include <QColor>
+#include <QObject>
+#include <QList>
+#include <QMutex>
+
+static const QColor segmentColors[] = 
+{       
+        Qt::darkGreen,
+        Qt::blue,
+        Qt::red,
+        Qt::darkYellow,
+        Qt::darkCyan,
+        Qt::darkMagenta,
+        Qt::black,
+        Qt::darkBlue,
+        Qt::darkRed
+};
+
+class SeekSegment
+{
+public:
+    SeekSegment( seeksegment_t *seeksegment )
+    {
+        i_time_offset = seeksegment->i_time_offset;
+        i_byte_offset = seeksegment->i_byte_offset;
+        i_time_duration = seeksegment->i_time_duration;
+        i_byte_duration = seeksegment->i_byte_duration;
+        qs_layerName = QString::fromUtf8( seeksegment->psz_layerName );
+        eventNum = seeksegment->eventNum;
+        qs_text = QString::fromUtf8( seeksegment->psz_text );
+        i_layer_id = seeksegment->i_layer_id;
+    };
+    int64_t i_byte_offset;
+    int64_t i_time_offset;
+    int64_t i_byte_duration;
+    int64_t i_time_duration;
+    QString qs_layerName;
+    int     eventNum;
+    QString qs_text;
+    int     i_layer_id;
+
+    //ROIY - TODO - Why do we need this class?
+};
+
+class SeekSegments : public QObject
+{
+    Q_OBJECT
+public:
+    SeekSegments( QObject *, intf_thread_t * );
+    ~SeekSegments();
+    QList<SeekSegment> const getSegments();
+    bool access() { return listMutex.tryLock( 100 ); }
+    void release() { listMutex.unlock(); }
+    bool jumpTo( int );
+    char* psz_layerName;
+
+public slots:
+    void update( bool isMain );
+    void updateMain();
+
+private:
+    QList<SeekSegment> segmentsList;
+    QMutex listMutex;
+    intf_thread_t *p_intf;
+};
+
+#endif // SEEKSEGMENTS_HPP
diff --git a/modules/gui/qt4/main_interface.cpp b/modules/gui/qt4/main_interface.cpp
index cab3433..c92ae76 100644
--- a/modules/gui/qt4/main_interface.cpp
+++ b/modules/gui/qt4/main_interface.cpp
@@ -332,12 +332,16 @@ void MainInterface::recreateToolbars()
 
     delete controls;
     delete inputC;
+	delete metaC;
 
     controls = new ControlsWidget( p_intf, b_adv, this );
     inputC = new InputControlsWidget( p_intf, this );
+	metaC = new MetaLayerControlsWidget( p_intf, this );
+
     mainLayout->insertWidget( 2, inputC );
     mainLayout->insertWidget( settings->value( "MainWindow/ToolbarPos", 0 ).toInt() ? 0: 3,
                               controls );
+	mainLayout->insertWidget( 3, metaC );
 
     if( fullscreenControls )
     {
@@ -417,6 +421,10 @@ void MainInterface::createMainWidget( QSettings *creationSettings )
         creationSettings->value( "MainWindow/ToolbarPos", 0 ).toInt() ? 0: 3,
         controls );
 
+	metaC = new MetaLayerControlsWidget( p_intf, this );
+	mainLayout->insertWidget( 3, metaC );
+
+
     /* Visualisation, disabled for now, they SUCK */
     #if 0
     visualSelector = new VisualSelector( p_intf );
diff --git a/modules/gui/qt4/main_interface.hpp b/modules/gui/qt4/main_interface.hpp
index bb413fc..e31afbc 100644
--- a/modules/gui/qt4/main_interface.hpp
+++ b/modules/gui/qt4/main_interface.hpp
@@ -49,6 +49,7 @@ class VisualSelector;
 class AdvControlsWidget;
 class ControlsWidget;
 class InputControlsWidget;
+class MetaLayerControlsWidget;
 class FullscreenControllerWidget;
 class SpeedControlWidget;
 class QVBoxLayout;
@@ -136,6 +137,8 @@ private:
     QVBoxLayout         *mainLayout;
     ControlsWidget      *controls;
     InputControlsWidget *inputC;
+	MetaLayerControlsWidget *metaC;
+    //MetaLayerControlsWidget *metaC2;  //ROIY remove?
     FullscreenControllerWidget *fullscreenControls;
 
     /* Widgets */
diff --git a/modules/gui/qt4/styles/metastyle.cpp b/modules/gui/qt4/styles/metastyle.cpp
new file mode 100644
index 0000000..ffaa896
--- /dev/null
+++ b/modules/gui/qt4/styles/metastyle.cpp
@@ -0,0 +1,275 @@
+/*****************************************************************************
+ * metastyle.cpp : meta slider style
+ ****************************************************************************
+ * Copyright (C) 2011-2012 VLC authors and VideoLAN
+ *
+ * Authors: Ludovic Fauvet <etix@videolan.org>
+ *
+ * This program is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation; either version 2 of the License, or
+ * (at your option) any later version.
+ *
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, write to the Free Software
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston MA 02110-1301, USA.
+ *****************************************************************************/
+
+#include "metastyle.hpp"
+#include "util/input_slider.hpp"
+#include "adapters/seekpoints.hpp"
+#include "adapters/seeksegments.hpp"
+
+#include <QWindowsStyle>
+#include <QStyleOptionSlider>
+#include <QPainter>
+#include <QDebug>
+#include <vector>
+
+#define RADIUS 3
+#define CHAPTERSSPOTSIZE 3
+
+int MetaStyle::pixelMetric( PixelMetric metric, const QStyleOption *option, const QWidget *widget ) const
+{
+    const QStyleOptionSlider *slider;
+
+    if ( metric == PM_SliderLength && ( slider = qstyleoption_cast<const QStyleOptionSlider *>( option ) ) )
+        return slider->rect.height();
+    else
+        return QWindowsStyle::pixelMetric( metric, option, widget );
+}
+
+void MetaStyle::drawComplexControl( ComplexControl cc, const QStyleOptionComplex *option, QPainter *painter, const QWidget *widget ) const
+{
+    if( cc == CC_Slider )
+    {
+        painter->setRenderHints( QPainter::Antialiasing );
+
+        if ( const QStyleOptionSlider *slider = qstyleoption_cast<const QStyleOptionSlider *>( option ) )
+        {
+            const MetaSlider *metaSlider = qobject_cast<const MetaSlider*>( widget );
+            qreal sliderPos = -1;
+
+            /* Get the needed subcontrols to draw the slider */
+            QRect groove = subControlRect(CC_Slider, slider, SC_SliderGroove, widget);
+            QRect handle = subControlRect(CC_Slider, slider, SC_SliderHandle, widget);
+
+            /* Adjust the size of the groove so the handle stays centered */
+            groove.adjust( handle.width() / 2, 0, -handle.width() / 2, 0 );
+
+            /* Reduce the height of the groove */
+            // Note: in the original 2.0.0 code, the groove had a height of 9px and to
+            // comply with the original style (but still allow the widget to expand) I
+            // had to remove 1 from the rect bottom.
+            groove.adjust( 0, (qreal)groove.height() / 3.7, 0, (qreal)-groove.height() / 3.7 - 1 );
+
+            if ( ( slider->subControls & SC_SliderGroove ) && groove.isValid() )
+            {
+                //sliderPos = ( ( (qreal)groove.width() ) / (qreal)slider->maximum )
+                //      * (qreal)slider->sliderPosition;  ROIY - DISABLED
+
+                /* set the background color and gradient */
+                QColor backgroundBase( slider->palette.window().color() );
+                QLinearGradient backgroundGradient( 0, 0, 0, slider->rect.height() );
+                backgroundGradient.setColorAt( 0.0, backgroundBase.darker( 140 ) );
+                backgroundGradient.setColorAt( 1.0, backgroundBase );
+
+                /* set the foreground color and gradient */
+                //QColor foregroundBase( 0, 153, 0 );
+                
+
+                /* draw a slight 3d effect on the bottom */
+                painter->setPen( QColor( 230, 230, 230 ) );
+                painter->setBrush( Qt::NoBrush );
+                painter->drawRoundedRect( groove.adjusted( 0, 2, 0, 0 ), RADIUS, RADIUS );
+
+                /* draw background */
+                painter->setPen( Qt::NoPen );
+                painter->setBrush( backgroundGradient );
+                painter->drawRoundedRect( groove, RADIUS, RADIUS );
+
+                /* adjusted foreground rectangle */
+                //QRect valueRect = groove.adjusted( 2000, 1, -1, 0 );
+                sliderPos = (qreal)groove.width() * 0.25;
+                QRect valueRect = groove.adjusted( 100, 1, -1, 0 );
+
+                valueRect.setWidth( sliderPos );
+
+                
+                if ( metaSlider->events )
+                {
+                    QColor foregroundBase;
+                    QList<SeekSegment> segmentsList = metaSlider->events->getSegments();
+                    int id = metaSlider->iEventLayersId;
+                    
+                    if ( id != -1 )     /* Secondary metaslider, color always the same */
+                        foregroundBase = segmentColors[id%9];
+                    
+                    foreach( SeekSegment segment, segmentsList )
+                    {
+                        if ( id == -1 )
+                            foregroundBase = segmentColors[segment.i_layer_id%9];   /* Colors repeat after 9 */
+                        
+                        QLinearGradient foregroundGradient( 0, 0, 0, groove.height() );
+                        foregroundGradient.setColorAt( 0.0,  foregroundBase );
+                        foregroundGradient.setColorAt( 1.0,  foregroundBase.darker( 125 ) );
+
+                        // draw Meta foreground
+                        painter->setPen( Qt::NoPen );
+                        painter->setBrush( foregroundGradient );
+
+                        qreal startPoint = (qreal)(segment.i_time_offset) / 1000000 / metaSlider->inputLength * groove.width();
+                        qreal duration = (qreal)(segment.i_time_duration) / 1000000 / metaSlider->inputLength * groove.width();
+                        
+                        QRect eventRect = groove.adjusted(startPoint , 1, -1, 0 );
+                        eventRect.setWidth( duration );
+                        
+                        painter->drawRoundedRect( eventRect, RADIUS, RADIUS );
+                    }
+                }
+
+
+                /* draw buffering overlay */
+                if ( metaSlider && metaSlider->f_buffering < 1.0 )
+                {
+                    QRect innerRect = groove.adjusted( 1, 1,
+                                        groove.width() * ( -1.0 + metaSlider->f_buffering ) - 1, 0 );
+                    QColor overlayColor = QColor( "Orange" );
+                    overlayColor.setAlpha( 128 );
+                    painter->setBrush( overlayColor );
+                    painter->drawRoundedRect( innerRect, RADIUS, RADIUS );
+                }
+            }
+
+            if ( slider->subControls & SC_SliderTickmarks ) {
+                QStyleOptionSlider tmpSlider = *slider;
+                tmpSlider.subControls = SC_SliderTickmarks;
+                QWindowsStyle::drawComplexControl(cc, &tmpSlider, painter, widget);
+            }
+
+            if ( slider->subControls & SC_SliderHandle && handle.isValid() 
+                && ( slider->sliderPosition > slider->minimum ) && ( slider->sliderPosition <= slider->maximum ) )
+            {
+                /* Useful for debugging */
+                //painter->setBrush( QColor( 0, 0, 255, 150 ) );
+                //painter->drawRect( handle );
+
+                //if ( option->state & QStyle::State_MouseOver || (metaSlider && metaSlider->isAnimationRunning() ) )
+                //{
+                    QPalette p = slider->palette;
+
+                    /* draw chapters tickpoints */
+                    if ( metaSlider->chapters && metaSlider->inputLength && groove.width() )
+                    {
+                        QColor background = p.color( QPalette::Active, QPalette::Window );
+                        QColor foreground = p.color( QPalette::Active, QPalette::WindowText );
+                        foreground.setHsv( foreground.hue(),
+                                        ( background.saturation() + foreground.saturation() ) / 2,
+                                        ( background.value() + foreground.value() ) / 2 );
+                        if ( slider->orientation == Qt::Horizontal ) /* TODO: vertical */
+                        {
+                            QList<SeekPoint> points = metaSlider->chapters->getPoints();
+                            foreach( SeekPoint point, points )
+                            {
+                                int x = groove.x() + point.time / 1000000.0 / metaSlider->inputLength * groove.width();
+                                painter->setPen( foreground );
+                                painter->setBrush( Qt::NoBrush );
+                                painter->drawLine( x, slider->rect.height(), x, slider->rect.height() - CHAPTERSSPOTSIZE );
+                            }
+                        }
+                    }
+
+                    /* draw handle */
+                    if ( ( sliderPos != -1 ) && ( metaSlider->iEventLayersId == -1 ) ) //Only draw for main metaslider. ROIY option->state & QStyle::State_Enabled && 
+                    {
+                        QSize hSize = QSize( handle.height(), handle.height() ) - QSize( 6, 6 );;
+                        QPoint pos = QPoint( handle.center().x() - ( hSize.width() / 2 ), handle.center().y() - ( hSize.height() / 2 ) );
+
+                        QPoint shadowPos( pos - QPoint( 2, 2 ) );
+                        QSize sSize( QSize( handle.height(), handle.height() ) - QSize( 2, 2 ) );
+
+                        /* prepare the handle's gradient */
+                        QLinearGradient handleGradient( 0, 0, 0, hSize.height() );
+                        QLinearGradient secondaryHandleGradient( 0, 0, 0, hSize.height() );
+                        QColor handleColor, secondaryHandleColor;
+
+                        if ( metaSlider->getEditState() != disabled )
+                        {
+                            if ( metaSlider->getEditState() == segmentEnd )
+                            {
+                                handleColor = Qt::red;
+                                secondaryHandleColor = Qt::green;
+                            }
+                            else 
+                            {
+                                handleColor = Qt::green;
+                                secondaryHandleColor = Qt::red;
+                            }
+                            
+                            secondaryHandleGradient.setColorAt( 0.0, secondaryHandleColor.lighter( 120 ) );
+                            secondaryHandleGradient.setColorAt( 0.9, secondaryHandleColor.darker( 120 ) );  
+                        }
+                        else
+                            handleColor = p.window().color();
+
+                        handleGradient.setColorAt( 0.0, handleColor.lighter( 120 ) );
+                        handleGradient.setColorAt( 0.9, handleColor.darker( 120 ) );
+
+                        /* prepare the handle's shadow gradient */
+                        QColor shadowBase = p.shadow().color();
+                        if( shadowBase.lightness() > 100 )
+                            shadowBase = QColor( 60, 60, 60 ); // Palette's shadow is too bright
+                        QColor shadowDark( shadowBase.darker( 150 ) );
+                        QColor shadowLight( shadowBase.lighter( 180 ) );
+                        shadowLight.setAlpha( 50 );
+
+                        QRadialGradient shadowGradient( shadowPos.x() + ( sSize.width() / 2 ),
+                                                        shadowPos.y() + ( sSize.height() / 2 ),
+                                                        qMax( sSize.width(), sSize.height() ) / 2 );
+                        shadowGradient.setColorAt( 0.4, shadowDark );
+                        shadowGradient.setColorAt( 1.0, shadowLight );
+
+                        painter->setPen( Qt::NoPen );
+                        if ( metaSlider != NULL )
+                            painter->setOpacity( metaSlider->mHandleOpacity );
+
+                        /* draw the handle's shadow */
+                        painter->setBrush( shadowGradient );
+                        painter->drawEllipse( shadowPos.x(), shadowPos.y() + 1, sSize.width(), sSize.height() );
+
+                        if ( metaSlider->getEditState() != disabled )
+                        {
+                            qreal staticSegmentLoc;
+
+                            /* draw the secondary handle's shadow */
+                            if ( metaSlider->getEditState() == segmentEnd ) 
+                                staticSegmentLoc = ( ( (qreal)groove.width() ) / (qreal)slider->maximum ) * (qreal)metaSlider->eventStartX;
+                            else
+                                staticSegmentLoc = ( ( (qreal)groove.width() ) / (qreal)slider->maximum ) * (qreal)metaSlider->eventEndX;
+
+                            painter->drawEllipse( staticSegmentLoc - 2, shadowPos.y() + 1, sSize.width(), sSize.height() );
+
+                            /* draw the secondary handle */
+                            painter->setBrush( secondaryHandleGradient );
+                            painter->drawEllipse( staticSegmentLoc, pos.y(), hSize.width(), hSize.height() );
+                        }
+
+                        /* finally draw the handle */
+                        painter->setBrush( handleGradient );
+                        painter->drawEllipse( pos.x(), pos.y(), hSize.width(), hSize.height() );
+                    }
+                //}
+            }
+        }
+    }
+    else
+    {
+        qWarning() << "MetaStyle: Drawing an unmanaged control";
+        QWindowsStyle::drawComplexControl( cc, option, painter, widget );
+    }
+}
diff --git a/modules/gui/qt4/styles/metastyle.hpp b/modules/gui/qt4/styles/metastyle.hpp
new file mode 100644
index 0000000..e3aa7d5
--- /dev/null
+++ b/modules/gui/qt4/styles/metastyle.hpp
@@ -0,0 +1,38 @@
+/*****************************************************************************
+ * metastyle.hpp : Meta slider style
+ ****************************************************************************
+ * Copyright (C) 2011-2012 VLC authors and VideoLAN
+ *
+ * Authors: Ludovic Fauvet <etix@videolan.org>
+ *
+ * This program is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation; either version 2 of the License, or
+ * (at your option) any later version.
+ *
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, write to the Free Software
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston MA 02110-1301, USA.
+ *****************************************************************************/
+
+#ifndef METASTYLE_HPP
+#define METASTYLE_HPP
+
+#include <QWindowsStyle>
+
+
+class MetaStyle : public QWindowsStyle
+{
+    Q_OBJECT
+public:
+    MetaStyle() { }
+    virtual int pixelMetric(PixelMetric metric, const QStyleOption * option = 0, const QWidget * widget = 0) const;
+    virtual void drawComplexControl(ComplexControl cc, const QStyleOptionComplex *opt, QPainter *p, const QWidget *widget) const;
+};
+
+#endif // METASTYLE_HPP
diff --git a/modules/gui/qt4/util/input_slider.cpp b/modules/gui/qt4/util/input_slider.cpp
index 3a92a78..52e1dac 100644
--- a/modules/gui/qt4/util/input_slider.cpp
+++ b/modules/gui/qt4/util/input_slider.cpp
@@ -1,5 +1,5 @@
 /*****************************************************************************
- * input_slider.cpp : VolumeSlider and SeekSlider
+ * input_slider.cpp : VolumeSlider, SeekSlider and MetaSlider
  ****************************************************************************
  * Copyright (C) 2006-2011 the VideoLAN team
  * $Id$
@@ -648,3 +648,259 @@ void SoundSlider::paintEvent( QPaintEvent *e )
     painter.end();
     e->accept();
 }
+
+/* MetaSlider - used to display and navigate Enhancement Data: Events and Annotations */
+
+MetaSlider::MetaSlider( Qt::Orientation q, QWidget *_parent, intf_thread_t * _p_intf, bool _classic )
+          : SeekSlider( q, _parent, _classic ), p_intf( _p_intf )
+{
+    alternativeStyle = new MetaStyle;
+    setStyle( alternativeStyle );
+    events = NULL;
+    iEventLayersId = -1;
+    editState = disabled;
+    eventStartX = -1;
+    eventEndX = -1;
+
+    /* Annotation Tooltip bubble */
+    mAnnotationTooltip = new AnnotationTooltip( this );
+    mAnnotationTooltip->setMouseTracking( true );
+
+    mAnnotationTooltip->installEventFilter( this );
+}
+
+MetaSlider::~MetaSlider()
+{
+    delete events; 
+}
+
+/***
+ * \brief Sets the events seeksegments adapter
+ *
+ * \params SeekSegments initialized with current intf thread
+***/
+void MetaSlider::setEvents( SeekSegments *events_ )
+{
+    delete events;
+    events = events_;
+    events->setParent( this );
+}
+
+void MetaSlider::toggleEditing()
+{
+    if ( editState == disabled )
+        editState = segmentEnd;
+    else
+        editState = disabled;
+
+    if ( editState != disabled )
+    {
+        eventStartX = eventEndX = value(); //sliderPosition//+ ( handleLength() / 2 );//ROIY (float)( value() ) / 1000.0;
+    }
+    else
+    {
+        eventStartX = eventEndX = -1;
+    }
+    //msg_Err( p_intf, "ROIY VAL=%d, %d" ,eventStartX, sliderPosition() );
+    repaint();
+}
+
+EditState MetaSlider::getEditState() const
+{
+    return editState;
+}
+
+void MetaSlider::enterEvent( QEvent * )
+{
+    /* Don't show the tooltip if the slider is disabled or a menu is open */
+    if( isEnabled() && inputLength > 0 && !qApp->activePopupWidget() )
+    {
+        //mTimeTooltip->show();
+        mAnnotationTooltip->show();
+    }
+}
+
+void MetaSlider::leaveEvent( QEvent * )
+{
+    //mTimeTooltip->hide();
+    mAnnotationTooltip->hide();
+}
+
+void MetaSlider::mouseMoveEvent( QMouseEvent *event )
+{
+    if ( !isEnabled() ) return event->accept();
+
+    if( isSliding )
+    {
+        setValue( QStyle::sliderValueFromPosition( MINIMUM, MAXIMUM, event->x() - handleLength() / 2, width() - handleLength(), false) );
+        emit sliderMoved( value() );
+    }
+
+    /* Tooltip */
+    if ( inputLength > 0 )
+    {
+        int margin = handleLength() / 2;
+        int posX = qMax( rect().left() + margin, qMin( rect().right() - margin, event->x() ) );
+
+        QString chapterLabel;
+
+        if ( orientation() == Qt::Horizontal ) /* TODO: vertical */
+        {
+                QList<SeekPoint> points = chapters->getPoints();
+                int i_selected = -1;
+                bool b_startsnonzero = false;
+                if ( points.count() > 0 )
+                    b_startsnonzero = ( points.at(0).time > 0 );
+                for( int i = 0 ; i < points.count() ; i++ )
+                {
+                    int x = points.at(i).time / 1000000.0 / inputLength * size().width();
+                    if ( event->x() >= x )
+                        i_selected = i + ( ( b_startsnonzero )? 1 : 0 );
+                }
+                if ( i_selected >= 0 && i_selected < points.size() )
+                    chapterLabel = points.at( i_selected ).name;
+        }
+
+        QPoint target( event->globalX() - ( event->x() - posX ),
+                  QWidget::mapToGlobal( QPoint( 0, 0 ) ).y() );
+        if( likely( size().width() > handleLength() ) ) {
+            //secstotimestr( psz_length, ( ( posX - margin ) * inputLength ) / ( size().width() - handleLength() ) );
+            //mTimeTooltip->setTip( target, psz_length, chapterLabel );
+
+            /* display annotation */
+            input_thread_t *p_input = playlist_CurrentInput( THEPL );
+            seeksegment_t *segments = NULL;
+            int64_t searchTime = ( ( ( posX - margin ) * inputLength * (int64_t)1000000 ) / ( size().width() - handleLength() ) );
+
+            if ( input_Control( p_input, INPUT_GET_ANNOTATIONS, &segments, NULL, searchTime, iEventLayersId )
+                != VLC_SUCCESS )
+            {
+                vlc_object_release( p_input );
+                return;
+            }
+            
+            if ( segments )
+            {
+                /*Check layer is valid and annotation matches slider layer */
+                if ( segments->psz_layerName )  
+                {
+                    mAnnotationTooltip->setTip( target, QString(segments->psz_layerName), segments->eventNum , QString(segments->psz_text) );
+                    mAnnotationTooltip->show();
+                }  
+
+                vlc_seeksegment_Delete( segments );
+            }
+            else
+                mAnnotationTooltip->hide();
+
+            vlc_object_release( p_input );
+        }
+    }
+    event->accept();
+}
+
+void MetaSlider::mousePressEvent( QMouseEvent *event )
+{
+        /* Right-click */
+    if ( !isEnabled() ||
+         ( event->button() != Qt::LeftButton && event->button() != Qt::MidButton )
+       )
+    {
+        QSlider::mousePressEvent( event );
+        return;
+    }
+
+    setValue( QStyle::sliderValueFromPosition( MINIMUM, MAXIMUM, event->x() - handleLength() / 2, width() - handleLength(), false ) );
+
+    /* Handle editing case for switching active markers */ 
+    int markerLocation;
+
+    if ( editState != disabled ) 
+    {
+        if ( editState == segmentEnd )
+            markerLocation = eventStartX - handleLength() / 2;
+        else
+            markerLocation = eventEndX - handleLength() / 2;
+            
+        if ( abs( value() - markerLocation ) <= ( handleLength() / 2) )    //User clicked on the other marker, switch control
+        {
+            if ( editState == segmentEnd )
+            {
+                editState = segmentStart;
+                emit switchEdit( false, eventEndX );    //ROIY - remove evenendx
+            }
+            else
+            {
+                editState = segmentEnd;
+                emit switchEdit( true, eventStartX );
+            }
+            repaint();  //ROIY needed?
+        }
+    }
+
+    isSliding = true;
+    emit sliderMoved( value() );
+    event->accept();
+}
+
+void MetaSlider::setPosition( float pos, int64_t time, int length )
+{
+    SeekSlider::setPosition( pos, time, length );
+
+    if ( editState == disabled )
+        return;
+
+    if ( editState == segmentEnd )
+    {
+        eventEndX = (int)( pos * 1000.0 );//msg_Err( p_intf, "ROIY SETPOS float =%f, eventEndx=%d, slidPos=%d , val=%d" , pos, eventEndX, sliderPosition(), value() );
+
+        if ( eventEndX < eventStartX )  /* Maintain correct order */
+            eventStartX = eventEndX;
+    }
+    else if ( editState == segmentStart )
+    {
+        eventStartX = (int)( pos * 1000.0 );
+
+        if ( eventEndX < eventStartX )  /* Maintain correct order */
+            eventEndX = eventStartX;
+    }
+}
+
+void MetaSlider::updatePos()
+{
+    SeekSlider::updatePos();
+
+    if ( editState == disabled )
+        return;
+
+    if ( editState == segmentEnd )
+    {
+        eventEndX = value();// + ( handleLength() / 2)  ROIY
+
+        if ( eventEndX < eventStartX )  /* Maintain correct order */
+            eventStartX = eventEndX;
+    }
+    else if ( editState == segmentStart )
+    {
+        eventStartX = value();// + ( handleLength() / 2); ROIY
+
+        if ( eventEndX < eventStartX )  /* Maintain correct order */
+            eventEndX = eventStartX;
+    }
+
+    //msg_Err( p_intf, "ROIY eventEndx=%d, eventStart=%d , slidPos=%d , val=%d" ,eventEndX, eventStartX, sliderPosition(), value() );
+}
+
+void MetaSlider::setEventTimes( int64_t startTime, int64_t endTime )
+{
+    if ( !inputLength )
+        return;
+
+    if ( startTime != -1 )
+        eventStartX = startTime / 1000000.0 / inputLength * ( size().width() - handleLength() );
+
+    if ( endTime != -1 )
+        eventEndX = endTime / 1000000.0 / inputLength * ( size().width() - handleLength() );
+
+    //msg_Err( p_intf, "ROIY SETEVENTTIMES eventEndx=%d, slidPos=%d , val=%d" ,eventEndX, sliderPosition(), value() );
+}
diff --git a/modules/gui/qt4/util/input_slider.hpp b/modules/gui/qt4/util/input_slider.hpp
index 00b7b19..07963a3 100644
--- a/modules/gui/qt4/util/input_slider.hpp
+++ b/modules/gui/qt4/util/input_slider.hpp
@@ -32,10 +32,14 @@
 
 #include <vlc_common.h>
 #include "timetooltip.hpp"
+#include "annotation_tooltip.hpp"
 #include "styles/seekstyle.hpp"
+#include "styles/metastyle.hpp"
+#include "adapters/seeksegments.hpp"
 
 #include <QSlider>
 #include <QPainter>
+#include <vector>
 
 #define MSTRTIME_MAX_SIZE 22
 
@@ -48,6 +52,13 @@ class QPropertyAnimation;
 class QStyleOption;
 class QCommonStyle;
 
+enum EditState
+{
+    disabled = -1,
+    segmentStart = 1,
+    segmentEnd = 2
+};
+
 /* Input Slider derived from QSlider */
 class SeekSlider : public QSlider
 {
@@ -76,16 +87,18 @@ protected:
     qreal handleOpacity() const;
     void setHandleOpacity( qreal opacity );
     int handleLength();
+    SeekPoints* chapters;
+    char psz_length[MSTRTIME_MAX_SIZE];               /* Used for the ToolTip */
+    
+    QCommonStyle *alternativeStyle;
+    TimeTooltip *mTimeTooltip;
+    int inputLength;                           /* InputLength that can change */
+	bool isSliding;        /* Whether we are currently sliding by user action */
 
 private:
-    bool isSliding;        /* Whether we are currently sliding by user action */
     bool isJumping;              /* if we requested a jump to another chapter */
-    int inputLength;                           /* InputLength that can change */
-    char psz_length[MSTRTIME_MAX_SIZE];               /* Used for the ToolTip */
     QTimer *seekLimitTimer;
-    TimeTooltip *mTimeTooltip;
     float f_buffering;
-    SeekPoints* chapters;
     bool b_classic;
     bool b_seekable;
     int mHandleLength;
@@ -98,7 +111,7 @@ private:
     QColor tickpointForeground;
     QColor shadowDark;
     QColor shadowLight;
-    QCommonStyle *alternativeStyle;
+
 
     /* Handle's animation */
     qreal mHandleOpacity;
@@ -111,15 +124,18 @@ public slots:
     void updateBuffering( float );
     void hideHandle();
 
+protected slots:
+    void updatePos();
+
 private slots:
     void startSeekTimer();
-    void updatePos();
 
 signals:
     void sliderDragged( float );
 
 
     friend class SeekStyle;
+    friend class MetaStyle;
 };
 
 /* Sound Slider inherited directly from QAbstractSlider */
@@ -165,4 +181,43 @@ private:
     void changeValue( int x ); /* Function to modify the value from pixel x() */
 };
 
+class MetaSlider : public SeekSlider
+{
+    Q_OBJECT
+public:
+    MetaSlider( Qt::Orientation q, QWidget *_parent = 0, struct intf_thread_t * = NULL, bool _classic = false );
+    ~MetaSlider();
+    void setEvents( SeekSegments * );
+    EditState getEditState() const;
+    SeekSegments* events;
+    int iEventLayersId;
+    int eventStartX;
+    int eventEndX;
+
+public slots:
+    void toggleEditing();
+    void setPosition( float pos, int64_t time, int length );
+    void setEventTimes( int64_t startTime, int64_t endTime );
+    void updatePos();
+
+protected:
+    virtual void enterEvent( QEvent * );
+    virtual void leaveEvent( QEvent * );
+    virtual void mouseMoveEvent( QMouseEvent * );
+    virtual void mousePressEvent( QMouseEvent * );
+    
+    AnnotationTooltip *mAnnotationTooltip;
+
+private:
+    struct intf_thread_t *p_intf;
+    EditState editState;
+
+    friend class MetaStyle;
+
+signals:
+    void switchEdit( bool, int64_t );
+
+};
+    
+
 #endif
-- 
1.8.3.msysgit.0

